--[[===================================================================================
Keybindings & MIDI Control of various functions useful for live performance.

Author: Dale Powell (Kazakore) <dj_kaza@hotmail.com>
Version: 0.2
===================================================================================]]--


-- *** >VARIABLES< ***
  local tool = renoise.tool()
  local rns = renoise.song
  local app = renoise.app()
  local tempox = 0.1 --Amount tempo is changed by with Increment/Decrement Tempo.
  local tnudge = 0.05 --Weighting amount for Nudge commands.
  local kp = os.clock()
  local tt = {0, 0, 0, 0} --Table used for Tap Tempo
  
-- *** >TAP TEMPO< ***

-- Tap
  function tap()
    tt[1] = tt[2]
    tt[2] = tt[3]
    tt[3] = tt[4]
    tt[4] = os.clock()
    local dif = tt[4] - tt[1]
    
    if dif < 0.6 or dif > 6 then return
    else rns().transport.bpm = 180/dif
    end
  end

-- MIDI Mapping
  tool:add_midi_mapping{
    name = "Kazakore's Live Functions:Tempo:Tap Tap [Trigger]",
    invoke = function(message)
      if (message:is_trigger()) then      
        tap()      
      end
    end
  }

-- Keybinding
  tool:add_keybinding {
    name = "Global:Kazakore's Tempo Functions:Tap Tempo",
    invoke = function() tap() end
  }
-- *** >TEMPO/TRACK NUDGE< ***

-- Nudge functions (brief tempo changes)
  local function nudge(dir)
    local bpm = rns().transport.bpm
    if dir == 1 and bpm > 949 then app:show_status("BPM too near extremes to nudge pitch") return
      elseif dir == -1 and bpm < 33.6 then app:show_status("BPM too near extremes to nudge pitch") return
      else local timer = os.clock()
      while timer > (os.clock() - 0.1) do
        rns().transport.bpm = bpm + rns().transport.bpm * tnudge * dir
      end
      rns().transport.bpm = bpm
    end
  end
  
-- MIDI Mappings
  tool:add_midi_mapping {
    name = "Kazakore's Live Functions:Tempo:Nudge Song Forward",
    invoke = function() nudge(1) end
  }
  tool:add_midi_mapping {
    name = "Kazakore's Live Functions:Tempo:Nudge Song Backward",
    invoke = function() nudge(-1) end
  }

-- Keybindings
  tool:add_keybinding {
    name = "Global:Kazakore's Tempo Functions:Nudge Song Forward",
    invoke = function() nudge(1) end
  }
  tool:add_keybinding {
    name = "Global:Kazakore's Tempo Functions:Nudge Song Backward",
    invoke = function() nudge(-1) end
  }
  

-- *** >TEMPO ADJUSTMENT< ***
    
-- Change and Display Tempo
  local function tempo (dir, mul)
    local bpm = rns().transport.bpm
    if dir == 1 and bpm > 999.8 then app:show_status("BPM too near extremes to increase tempo") return
      elseif dir == -1 and bpm < 32.2 then app:show_status("BPM too near extremes to reduce tempo") return
      elseif mul == 1 and bpm > 499 then app:show_status("BPM too high to double tempo") return
      elseif mul == -0.5 and bpm < 64 then app:show_status("BPM too low to half tempo") return
      else rns().transport.bpm = bpm + (tempox*dir) + (bpm*mul)
      bpm = rns().transport.bpm
      app:show_status("tempo is now "..bpm)
    end
  end
      
-- MIDI Mappings
  tool:add_midi_mapping{
    name = "Kazakore's Live Functions:Tempo:Increment Tempo [Trigger]",
    invoke = function(message)
      if (message:is_trigger()) then      
        tempo(1, 0)      
      end
    end
  }
  tool:add_midi_mapping{
    name = "Kazakore's Live Functions:Tempo:Decrement Tempo [Trigger]",
    invoke = function(message)
      if (message:is_trigger()) then      
        tempo(-1, 0)      
      end
    end
  }
  tool:add_midi_mapping{
    name = "Kazakore's Live Functions:Tempo:Double Tempo [Trigger]",
    invoke = function(message)
      if (message:is_trigger()) then      
        tempo(0, 1)      
      end
    end
  }
  tool:add_midi_mapping{
    name = "Kazakore's Live Functions:Tempo:Half Tempo [Trigger]",
    invoke = function(message)
      if (message:is_trigger()) then      
        tempo(0, -0.5)      
      end
    end
  }
  
-- Keybindings
  tool:add_keybinding {
    name = "Global:Kazakore's Tempo Functions:Increment Tempo",
    invoke = function() tempo(1, 0) end
  }
  tool:add_keybinding {
    name = "Global:Kazakore's Tempo Functions:Decrement Tempo",
    invoke = function() tempo(-1, 0) end
  }
  tool:add_keybinding {
    name = "Global:Kazakore's Tempo Functions:Double Tempo",
    invoke = function() tempo(0, 1) end
  }
  tool:add_keybinding {
    name = "Global:Kazakore's Tempo Functions:Half Tempo",
    invoke = function() tempo(0, -0.5) end
  }
  
  
-- *** >SEND TRACK AUTOMATION CLEAR< ***

-- Number of Send Tracks to create bindings for:
MAX_SENDS = 8

-- Main Clear Send function
local function clearST(send, pat)
  local tc = rns().sequencer_track_count
  local tr = tc + 1 + send  -- specify track index to function

  
  if tr > #rns().tracks then app:show_status("Send Track index higher than track count") return end
  
  if pat == "live" then
    local seql = rns().transport.song_length.sequence
      if seql < 2 then app:show_status("Sequence less than 2 patterns in length") return end
    local pt1 = rns().sequencer:pattern(seql-1)
    local pt2 = rns().sequencer:pattern(seql)
    for i=1, #rns():pattern(pt1):track(tr).automation do
      rns():pattern(pt1):track(tr).automation[i]:clear()
    end
    for i=1, #rns():pattern(pt2):track(tr).automation do
      rns():pattern(pt2):track(tr).automation[i]:clear()
  end

  elseif pat == "cur" then
    for i=1, #rns().selected_pattern:track(tr).automation do
      rns().selected_pattern:track(tr).automation[i]:clear()
    end
    
  elseif pat == "all" then
    for i=1, #rns().selected_pattern:track(tr).automation do
      for j=1, #rns().patterns do
        rns():pattern(j):track(tr).automation[i]:clear()
      end
    end
  end

end

-- Keybindings
for i = 1, MAX_SENDS do
  tool:add_keybinding {
    name = "Global:Kazakore's Send Track Automation Clear:Send  " .. tostring(i) .. " Clear ('Live' Patterns)",
    invoke = function() clearST(i, "live") end
  }
end
for i = 1, MAX_SENDS do
  tool:add_keybinding {
    name = "Global:Kazakore's Send Track Automation Clear:Send " .. tostring(i) .. " Clear (Current Pattern)",
    invoke = function() clearST(i, "cur") end
  }
end
for i = 1, MAX_SENDS do
  tool:add_keybinding {
    name = "Global:Kazakore's Send Track Automation Clear:Send " .. tostring(i) .. " Clear (All Patterns)",
    invoke = function() clearST(i, "all") end
  }
end

-- MIDI Mappings
for i = 1, MAX_SENDS do
  tool:add_midi_mapping {
    name = "Kazakore's Live Functions:Clear Send Track Automation:Send " .. tostring(i) .. " Clear ('Live' Patterns)",
    invoke = function() clearST(i, "live") end
  }
end
for i = 1, MAX_SENDS do
  tool:add_midi_mapping {
    name = "Kazakore's Live Functions:Clear Send Track Automation:Send " .. tostring(i) .. " Clear (Current Pattern)",
    invoke = function() clearST(i, "cur") end
  }
end
for i = 1, MAX_SENDS do
  tool:add_midi_mapping {
    name = "Kazakore's Live Functions:Clear Send Track Automation:Send " .. tostring(i) .. " Clear (Current Patterns)",
    invoke = function() clearST(i, "all") end
  }
end

--[
-- ***>JUMP TO SECTION IN SEQUENCE< ***

--Number of Sections to create shortcuts for
  MAX_SEQ = 8
  
-- Main Jump To Section function
  local function jump(jsq, mode)
  local jpat = 0

    for i = 1, jsq do
      for j = jpat+1, rns().transport.song_length.sequence do
        if rns().sequencer:sequence_is_start_of_section(j) == true then jpat=j break end
        if j == rns().transport.song_length.sequence then app:show_status("Reached end of Sequence list") return end
      end
    end
  -- Jump EditPos to Section X
    if mode == "edit" then
      local pos = rns().transport.edit_pos
      pos.sequence = jpat
      rns().transport.edit_pos = pos
  -- Queue Section X
    elseif mode == "play" then
      rns().transport:set_scheduled_sequence(jpat)
    end
  end
  
  local function jumpl(mode)
    if rns().transport.song_length.sequence <2 then app:show_status("Sequence too short") return end
    if mode == "edit" then
      local pos = rns().transport.edit_pos
      pos.sequence = rns().transport.song_length.sequence - 1
      rns().transport.edit_pos = pos
    elseif mode == "play" then
      rns().transport:set_scheduled_sequence(rns().transport.song_length.sequence - 1)
    end
  end
  
-- Keybings
  for i=1, MAX_SEQ do
    tool:add_keybinding {
      name = "Global:Kazakore's Section Navigation:Jump EditPos to Section " .. i,
      invoke = function() jump(i, "edit") end
    }
  end
  for i=1, MAX_SEQ do
    tool:add_keybinding {
      name = "Global:Kazakore's Section Navigation:Queue Section " .. i,
      invoke = function() jump(i, "play") end
    }
  end
  tool:add_keybinding {
      name = "Global:Kazakore's Section Navigation:Jump EditPos to 'Live' Patterns",
      invoke = function() jumpl("edit") end
  }
  tool:add_keybinding {
      name = "Global:Kazakore's Section Navigation:Queue 'Live' Patterns",
      invoke = function() jumpl("play") end
  }

-- MIDI Mappings
  for i=1, MAX_SEQ do
    tool:add_midi_mapping{
      name = "Kazakore's Live Functions:Sequence:Jump EditPos to Section " .. i,
      invoke = function(message)
        if (message:is_trigger()) then
          jump(i, "edit")
        end
      end
    }
  end
    for i=1, MAX_SEQ do
    tool:add_midi_mapping{
      name = "Kazakore's Live Functions:Sequence:Queue Section " .. i,
      invoke = function(message)
        if (message:is_trigger()) then
          jump(i, "play")
        end
      end
    }
  end
  

